@@ -9,7 +9,7 @@ import multiprocessing
 import socket
 import subprocess
 import sys
-import time
+import time as tm
 import platform
 import requests
 import randomname
@@ -72,6 +72,18 @@ def calculate_sha256(filename):
     return None
 
 
+def validate_timestamp(timestamp):
+    """
+    Validate a user-provided timestamp
+    """
+    try:
+        datetime.datetime.strptime(timestamp, '%Y-%m-%d %H:%M:%S.%f')
+    except ValueError:
+        return False
+
+    return True
+
+
 class Simvue(object):
     """
     Track simulation details based on token and URL
@@ -131,7 +143,7 @@ class Simvue(object):
 
         self._headers = {"Authorization": "Bearer %s" % token}
         self._name = name
-        self._start_time = time.time()
+        self._start_time = tm.time()
 
         self._worker = Worker(self._metrics_queue, self._events_queue, name, self._url, self._headers)
 
@@ -217,7 +229,7 @@ class Simvue(object):
 
         return False
 
-    def log_event(self, message):
+    def log_event(self, message, timestamp=None):
         """
         Write event
         """
@@ -231,6 +243,11 @@ class Simvue(object):
         data['run'] = self._name
         data['message'] = message
         data['timestamp'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
+        if timestamp is not None:
+            if validate_timestamp(timestamp):
+                data['timestamp'] = timestamp
+            else:
+                raise RuntimeError('Invalid timestamp format')
 
         try:
             self._events_queue.put(data, block=self._queue_blocking)
@@ -239,7 +256,7 @@ class Simvue(object):
 
         return True
 
-    def log_metrics(self, metrics):
+    def log_metrics(self, metrics, time=None, timestamp=None):
         """
         Write metrics
         """
@@ -255,8 +272,15 @@ class Simvue(object):
         data = {}
         data['run'] = self._name
         data['values'] = metrics
-        data['time'] = time.time() - self._start_time
+        data['time'] = tm.time() - self._start_time
+        if time is not None:
+            data['time'] = time
         data['timestamp'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')
+        if timestamp is not None:
+            if validate_timestamp(timestamp):
+                data['timestamp'] = timestamp
+            else:
+                raise RuntimeError('Invalid timestamp format')
         data['step'] = self._step
 
         self._step += 1
